| Safe Haskell | Safe-Inferred |
|---|---|
| Language | Haskell2010 |
Aztecs.ECS.Class
Documentation
class ECS (m :: Type -> Type) where Source #
Entity Component System (ECS) implementation.
Associated Types
type Entity (m :: Type -> Type) Source #
Entity identifier.
type Task (m :: Type -> Type) :: Type -> Type Source #
Task monad for running systems.
Methods
spawn :: Bundle (Entity m) m -> m (Entity m) Source #
Spawn a new entity with a Bundle of components.
insert :: Entity m -> Bundle (Entity m) m -> m () Source #
Insert a Bundle of components into an existing entity
(otherwise this will do nothing).
remove :: Entity m -> m () Source #
Remove an entity and its components.
task :: Task m a -> m a Source #
Run a Task.
Instances
| PrimMonad m => ECS (AztecsT cs m) Source # | |||||||||
Defined in Aztecs.Internal Associated Types
| |||||||||