aztecs-0.16.0: A modular game engine and Entity-Component-System (ECS) for Haskell.
Copyright(c) Matt Hunzinger 2025
LicenseBSD-style (see the LICENSE file in the distribution)
Maintainermatt@hunzinger.me
Stabilityprovisional
Portabilitynon-portable (GHC extensions)
Safe HaskellNone
LanguageHaskell2010

Aztecs.ECS.World.Bundle.Dynamic

Description

 
Synopsis

Documentation

newtype DynamicBundle (m :: Type -> Type) Source #

Dynamic bundle of components.

Constructors

DynamicBundle 

Fields

class MonoidDynamicBundle (m :: Type -> Type) a where Source #

Monoid bundle of dynamic components.

Methods

dynBundle :: Component m c => ComponentID -> c -> a Source #

Add a component to the bundle by its ComponentID.

dynBundleUntracked :: Component m c => ComponentID -> c -> a Source #

Add a component to the bundle by its ComponentID without running lifecycle hooks.

Instances

Instances details
Monad m => MonoidDynamicBundle m (BundleT m) Source # 
Instance details

Defined in Aztecs.ECS.World.Bundle

Monad m => MonoidDynamicBundle m (DynamicBundle m) Source # 
Instance details

Defined in Aztecs.ECS.World.Bundle.Dynamic